/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#if !defined(AFX_DELAYEDDELETE_H__BF228830_D633_439E_9E01_A9B497B2FAE0__INCLUDED_)
#define AFX_DELAYEDDELETE_H__BF228830_D633_439E_9E01_A9B497B2FAE0__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <list>
#include <vector>

#include "KomodiaSettings.h"

KOMODIA_NAMESPACE_START

class CGenericCriticalSection;
class CPeriodicThread;

class CDelayedDelete  
{
public:
	//The delete proc
	typedef void (*DeleteProc)(LPVOID pParam);
	typedef DeleteProc LPDeleteProc;
public:
	//Add an item to delete
	void AddItem(LPVOID lpItem,
				 LPDeleteProc pProc,
				 DWORD dwDeleteTimeout);

	//Ctor and Dtor
	CDelayedDelete();
	virtual ~CDelayedDelete();
private:
	//The delete strcuture
	typedef struct _DeleteStructure
	{
		LPDeleteProc	pProc;
		DWORD			dwExpireTime;
		LPVOID			lpItemToDelete;
	} DeleteStructure;

	//Our list of items to delete
	typedef std::list<DeleteStructure> DeleteList;

	//Our vector of deleted items (for thread safe)
	typedef std::vector<DeleteStructure> DeleteVector;
private:
	//No copy ctor
	CDelayedDelete(const CDelayedDelete& rDelete);

	//No assignment operator
	CDelayedDelete& operator=(const CDelayedDelete& rDelete);
	
	//The thread to manage the deletion
	static BOOL DeleteThreadProc(LPVOID lpParam);

	//Delete all remaining data
	void DeleteAll();

	//Our delete list
	DeleteList m_aData;

	//Our CS
	CGenericCriticalSection* m_pCS;

	//The periodic thread that manages the deletes
	CPeriodicThread* m_pThread;
};

KOMODIA_NAMESPACE_END

#endif // !defined(AFX_DELAYEDDELETE_H__BF228830_D633_439E_9E01_A9B497B2FAE0__INCLUDED_)
