/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#if !defined(AFX_TCPSOCKETASYNCBLOWFISH_H__71B6C66D_832F_4EE5_9257_C55AEBBF048B__INCLUDED_)
#define AFX_TCPSOCKETASYNCBLOWFISH_H__71B6C66D_832F_4EE5_9257_C55AEBBF048B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifdef _SSLInheritance
#include "TCPSocketAsyncSSL.h"
#else
#include "TCPSocketAsyncMsg.h"
#endif

#include "BlowFish.h"

KOMODIA_NAMESPACE_START

#ifdef _SSLInheritance
typedef CTCPSocketAsyncSSL BlowFishBaseClass;
#else
typedef CTCPSocketAsyncMsg BlowFishBaseClass;
#endif

class CTCPSocketAsyncBlowFish : public BlowFishBaseClass  
{
public:
	//Accept a connection, supply an already made socket
	virtual BOOL Accept(CTCPSocket* pNewSocket);

	//Set/Get the key phrase
	void SetKey(const std::string& rKey);
	const std::string& GetKey()const;

	//Receive a message
	//Returns number of bytes received or error code (-4 decryption problems!)
	//iMoreMessages - If more messages exist it will be 1
	//				  0 for no more messages
	//				  -size for buffer too small to accept message so abs() will give the size
	virtual int ReceiveMsg(char* pData,
						   int iSize,
						   int& iMoreMessages);

	//Send a message
	//Returns number of bytes sent, or -1 for error
	//-2 size mismatch
	virtual int SendMsg(const char* pData,
						int iSize);

	//Send the message but abort if event is set
	//Same error codes as the normal SendMsg
	//-3 for user abort
	virtual int SendMsg(const char* pData,
						int iSize,
						CGenericEvent* pStopEvent);

	//Ctor and Dtor
	CTCPSocketAsyncBlowFish();
	virtual ~CTCPSocketAsyncBlowFish();
private:
	//Encrypt a memory buffer (will overwrite data, make sure buffer is 20 bytes longer the size)
	unsigned long EncryptMemory(char* pBuffer,
								unsigned long ulSize)const;

	//Decrypt a memory buffer (will overwrite data)
	unsigned long DecryptMemory(char* pBuffer,
					  		    unsigned long ulSize)const;

	//Our key
	std::string m_sKey;

	//Our blowfish encrypt chain
	mutable blowfish::Block m_aEncryptChain;

	//Our blowfish decrypt chain
	mutable blowfish::Block m_aDecryptChain;
};

KOMODIA_NAMESPACE_END

#endif // !defined(AFX_TCPSOCKETASYNCBLOWFISH_H__71B6C66D_832F_4EE5_9257_C55AEBBF048B__INCLUDED_)
