/*
 *  Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Contact info:
 * -------------
 *
 * Site:					http://www.komodia.com
 * Main contact:			barak@komodia.com
 * For custom projects, 
 * consulting, or other
 * paid services:			sales@komodia.com
 */

#if !defined(AFX_TCPSOCKETASYNCMSG_H__F82FC905_951A_4FBE_ADE3_CC462DCC0EB3__INCLUDED_)
#define AFX_TCPSOCKETASYNCMSG_H__F82FC905_951A_4FBE_ADE3_CC462DCC0EB3__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "TCPSocketAsync.h"

#include <deque>

KOMODIA_NAMESPACE_START

class CGenericEvent;

class CTCPSocketAsyncMsg : public CTCPSocketAsync 
{
public:
	//Receive a message
	//Returns number of bytes received or error code
	//iMoreMessages - If more messages exist it will be 1
	//				  0 for no more messages
	//				  -size for buffer too small to accept message so abs() will give the size
	virtual int ReceiveMsg(char* pData,
						   int iSize,
						   int& iMoreMessages);

	//Send a message
	//Returns number of bytes sent, or -1 for error
	//-2 size mismatch
	virtual int SendMsg(const char* pData,
						int iSize);

	//Send the message but abort if event is set
	//Same error codes as the normal SendMsg
	//-3 for user abort
	virtual int SendMsg(const char* pData,
						int iSize,
						CGenericEvent* pStopEvent);

	//Ctor and Dtor
	CTCPSocketAsyncMsg();
	virtual ~CTCPSocketAsyncMsg();
private:
	//Our deque of data we cache
	typedef std::deque<char> CachedDeque;
private:
	//No Ctor
	CTCPSocketAsyncMsg(const CTCPSocketAsyncMsg& rSocket);

	//No assigment operator
	CTCPSocketAsyncMsg& operator=(const CTCPSocketAsyncMsg& rSocket);

	//Check if it's possible to extract a message, 0 for no, size for message size
	int CanExtractMsg(const CachedDeque& rData)const;

	//Extract a message, 0 for no, size for message size or -size for buffer too small
	int ExtractMsg(CachedDeque& rData,
				   char* pData,
				   int iSize)const;

	//Our internal version with recursion stop flag
	int InternalReceiveMsg(char* pData,
						   int iSize,
						   int& iMoreMessages,
						   bool bRecursed);

	//Our cached data
	CachedDeque m_aData;
};

KOMODIA_NAMESPACE_END

#endif // !defined(AFX_TCPSOCKETASYNCMSG_H__F82FC905_951A_4FBE_ADE3_CC462DCC0EB3__INCLUDED_)
